;Amiga Logo by Chris Covell
;-------------------
;Binary created using DAsm 2.12 running on an Amiga.

   PROCESSOR   6502

   ORG   $C000    ;16Kb PRG-ROM, 8Kb CHR-ROM

   dc.b  "Amiga! demo program by Chris Covell (ccovell@direct.ca)"
         ;Though thanks for the examples, Mark K!

Reset_Routine  SUBROUTINE
   cld         ;Clear decimal flag
   sei         ;Disable interrupts
.WaitV   lda $2002
   bpl .WaitV     ;Wait for vertical blanking interval
   ldx #$00
   stx $2000
   stx $2001      ;Screen display off, amongst other things
   dex
   txs         ;Top of stack at $1FF

;Clear (most of) the NES' WRAM. This routine is ripped from "Duck Hunt" - I should probably clear all
;$800 bytes.
   ldy #$06    ;To clear 7 x $100 bytes, from $000 to $6FF?
   sty $01        ;Store count value in $01
   ldy #$00
   sty $00
   lda #$00

.Clear   sta ($00),y    ;Clear $100 bytes
   dey
   bne .Clear

   dec $01        ;Decrement "banks" left counter
   bpl .Clear     ;Do next if >= 0


;Now we need to set up the palette and colour bar data.

   jsr   SetTables      ;Set up name and attribute table data
   ldy   #$00        ;Initial palette "position"
;Next instruction unnecessary since memory has just been cleared
  sty   $02         ;Store palette position
   jsr   SetPalette     ;Set initial palette data

;Enable vblank interrupts, etc.
   lda   #$80
   sta   $2000
   lda   #$0B        ;Screen on, sprites off, show leftmost 8 pixels, colour
   sta   $2001
;  cli            ;Enable interrupts(?)

;--- SET UP AUDIO ----------------------

   ldx   #%10110001
   stx   $4000
   stx   $4004
   stx   $4008
   ldx   #%00000000
   stx   $4001
   stx   $4005
   stx   $4009
   ldx   #%11111111
   stx   $4002
   stx   $4006
   stx   $400A
   ldx   #%00000111
   stx   $4003
   stx   $4007
   stx   $400B
   ldx   #%00011110
   stx   $4015

;--- END OF AUDIO ----------------------


;Now just loop forever?
.Loop jmp   .Loop


;Set the name ($2000) & attribute tables ($23C0)
;-----------------------------------------------

SetTables   SUBROUTINE

;Set the PPU memory address
   ldx   #$20  ;High byte of $2000
   stx   $2006
   ldx   #$00  ;Low byte of $2000
   stx   $2006

   ldy   #$0         ;Clear 256 bytes...
   ldx   #0
.SomeSpace  stx $2007
   dey
   bne   .SomeSpace

   ldy   $0          ;Start with char 0
   sty   $2007       ;Draw some filler
   sty   $2007       ;...again...

   ldy   #$E0
.Doaline   stx   $2007            ;Load into Name Table
   inx
   dey
   bne   .Doaline                 ;Until it loops back to 0???


   ldy   #$20
   sty   $2006
   ldy   #$D0
   sty   $2006

   stx   $2007
   inx
   stx   $2007

   ldy   #$20
   sty   $2006
   ldy   #$F0
   sty   $2006

   inx
   stx   $2007
   inx
   stx   $2007

   ldx   #$23  ;High byte of $23C0
   stx   $2006
   ldx   #$C0  ;Low byte of $23C0
   stx   $2006

   ldy   #$40    ;Clear entire attr space.
   ldx   #0
.AttrRow1   stx   $2007
   dey
   bne   .AttrRow1

   ldx   #$23  ;High byte of $23C0
   stx   $2006
   ldx   #$C0  ;Low byte of $23C0
   stx   $2006

   ldy   #$10    ;Clear entire attr space.
   ldx   #$55
.AttrRow2   stx   $2007
   dey
   bne   .AttrRow2


SetPalette  SUBROUTINE

;Set PPU memory address to $3F00
   ldx   #$3F     ;High byte of $3F00
   stx   $2006
   ldx   #$00     ;Low byte of $3F00
   stx   $2006

   ldy   #$20
   ldx   #$20
.ClearPal   stx   $2007
   dey
   bne   .ClearPal

   ldx   #$3F     ;High byte of $3F00
   stx   $2006
   ldx   #$01     ;Low byte of $3F01
   stx   $2006

   ldx   #$10
   stx   $2007
   ldx   #$0
   stx   $2007
   ldx   #$D
   stx   $2007

   ldx   #$3F     ;High byte of $3F00
   stx   $2006
   ldx   #$05     ;Low byte of $3F05
   stx   $2006

   ldx   #$35
   stx   $2007
   ldx   #$25
   stx   $2007
   ldx   #$15
   stx   $2007

   rts         ;Done!

NMI_Routine SUBROUTINE
   dec   $4002
   inc   $4006
   beq .Doup
.More   dec   $400A
   bne .Return
   dec   $4003
   dec   $400B
.Return
   ldx   #$0
   stx   $2005
   stx   $2005

   rti

.Doup inc   $4007
      jmp .More

IRQ_Routine       ;Dummy label
   rti

;That's all the code. Now we just need to set the vector table approriately.

   ORG   $FFFA,0
   dc.w  NMI_Routine
   dc.w  Reset_Routine
   dc.w  IRQ_Routine    ;Not used, just points to RTI


;The end.
